﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class BeamBuilder : Builder
	{
		int beamWidth, remineIterations, maxLength;
		public BeamBuilder(BinaryMatrix data, int remineIterations, int beamWidth, int maxLength): base(data)
		{
			this.remineIterations = remineIterations;
			this.beamWidth = beamWidth;
			this.maxLength = maxLength;
		}

		public override Classifier build(HashSet<int> trueRows)
		{
			//create first partition with all rows
			List<int> rows = new List<int>(data.M);
			for (int i = 0; i < data.M; i++)
			{
				rows.Add(i);
			}
			BeamPartition all = new BeamPartition(data, trueRows, rows);
			List<BeamPartition> partitioning = new List<BeamPartition>();
			partitioning.Add(all);

			//create classifier
			BeamClassifier classifier = new BeamClassifier();
			classifier.root = all.splitNode;

			int max = this.remineIterations;
			HashSet<int> attributes = new HashSet<int>();
			//for (int i = 0; i < data.N; i++)
			//	attributes.Add(i);
			foreach (int r in trueRows)
			{
				foreach (int a in data.getIndexesByValuePerRow(r, true))
					attributes.Add(a);
			}
			while (max > 0)
			{
				//find patterns in current partitioning
				List<BeamPattern> patterns = new List<BeamPattern>();
				foreach (BeamPartition p in partitioning)
				{
					BeamPattern pat = p.findBestPattern(beamWidth, maxLength, attributes);
					if (pat.gain > 0)
					{
						patterns.Add(pat);
					}
				}

				//split all parititions on new found patterns
				foreach (BeamPattern pat in patterns)
				{
				    List<BeamPartition> newPartitioning = new List<BeamPartition>();
				    foreach (BeamPartition p in partitioning)
				    {
				        List<BeamPartition> s = p.split(pat);
				        foreach (BeamPartition sp in s)
				        {
				            if (sp.rows.Count > 1)
				            {
				                newPartitioning.Add(sp);
				            }
				        }
				    }
				    partitioning = newPartitioning;
				}
				max--;
			}

			classifier.root.shrink();
			return classifier;
		}



	}
}
