﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class BeamNode
	{
		public BeamNode parent, left, right;
		public BeamPattern splitPattern;
		public bool classlabel;
		public int nt,nf;


		public bool classify(BitArray values)
		{
			if (splitPattern == null && left == null)
				return classlabel;
			if (splitPattern == null)
				return false;
			foreach (int i in splitPattern.pattern)
			{
				if (!values.get(i))
					return left.classify(values);
			}
			return right.classify(values);
		}

		public int[] count()
		{
			if (left == null && right == null)
				return new int[] { 0, 1 };
			int[] c = new int[] { 1, 0 };
			if (left != null)
			{
				int[] l = left.count();
				c[0] += l[0];
				c[1] += l[1];
			}
			if (right != null)
			{
				int[] r = right.count();
				c[0] += r[0];
				c[1] += r[1];
			}
			return c;
		}

		public override string ToString()
		{
			return base.ToString();
		}

		public bool[] shrink()
		{
			if (splitPattern == null)
			{
				bool[] values = new bool[2];
				if (classlabel)
					values[1] = true;
				else
					values[0] = true;
				return values;
			}
			bool[] lv = left.shrink();
			bool[] rv = right.shrink();

			if (lv[0] == true && lv[1] == false && rv[0] == true && rv[1] == false) //pure false
			{
				splitPattern = null;
				classlabel = false;
				left.parent = null;
				right.parent = null;
				left = null;
				right = null;
			}

			if (lv[0] == false && lv[1] == true && rv[0] == false && rv[1] == true) //pure true
			{
				splitPattern = null;
				classlabel = true;
				left.parent = null;
				right.parent = null;
				left = null;
				right = null;
			}

			return new bool[]{lv[0] || rv[0], lv[1] || rv[1]};
		}

		//public List<BeamNode> getExternalVertices()
		//{
		//}

		public void print(int d, TextWriter writer)
		{
			string line = new String(' ', d * 2);
			if (left == null && right == null)
			{
				line += "leaf node, size:("+nt+"/"+nf+"), class: " + classlabel;
			}
			else
			{
				line += "internal node, size:(" + nt + "/" + nf + "), pattern: {" + splitPattern + "}";
			}
			writer.WriteLine(line);
			if (left != null)
				left.print(d + 2, writer);
			if (right != null)
				right.print(d + 2, writer);
		}

	}
}
