﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	public class NBClassifier : Classifier
	{
		List<HashSet<int>> pin;
		List<List<HashSet<int>>> pout;
		HashSet<int> attributes;

		public NBClassifier(List<HashSet<int>> pin, List<List<HashSet<int>>> pout)
		{
			this.pin = pin;
			this.pout = pout;

			//save set off all used attributes
			attributes = new HashSet<int>();
			foreach (HashSet<int> set in pin)
				foreach (int i in set)
					attributes.Add(i);

			foreach (List<HashSet<int>> list in pout)
				foreach (HashSet<int> set in list)
					foreach (int i in set)
						attributes.Add(i);
		}

		public int CountAttributes
		{
			get
			{
				return attributes.Count;
			}
		}

		public int CountPatterns
		{
			get
			{
				return pin.Count;
			}
		}

		public int CountTotalPatternLength
		{
			get
			{
				//avg pattern lenght
				int total = 0;
				for (int i = 0; i < pin.Count; i++)
				{
					total += pin[i].Count;
					foreach (HashSet<int> set in pout[i])
						total += set.Count;
				}
				return total;
			}
		}

		public string getInformation()
		{
			//involved attributes
			HashSet<int> attributes = new HashSet<int>();
			foreach(HashSet<int> set in pin)
				foreach(int i in set)
					attributes.Add(i);

			foreach(List<HashSet<int>> list in pout)
				foreach(HashSet<int> set in list)
					foreach(int i in set)
						attributes.Add(i);

			//avg pattern lenght
			double total = 0;
			for (int i = 0; i < pin.Count; i++)
			{
				total += pin[i].Count;
				foreach (HashSet<int> set in pout[i])
					total += set.Count;
			}
			return CountPatterns + ";" + CountAttributes + ";" + CountTotalPatternLength+" ; "+(CountTotalPatternLength * 1.0 / CountPatterns+" ; "+Complexity);
		}

		public override bool classify(BitArray values, int row)
		{
			for (int i = 0; i < pin.Count; i++)
			{
				HashSet<int> setin = pin[i];
				List<HashSet<int>> setout = pout[i];
				bool valid = true;
				foreach (int v in setin)
				{
					if (!values.get(v))
					{
						valid = false;
						break;
					}
				}
				if (!valid)
					continue;
				
				foreach (HashSet<int> set in setout)
				{
					int counter = 0;
					foreach (int v in set)
					{
						if (values.get(v))
						{
							counter++;
						}
					}
					if (counter == set.Count)
					{
						valid = false;
						break;
					}	
				}
				if (valid)
					return true;
			}
			return false;
		}

		public double Complexity
		{
			get
			{
				return attributes.Count;
			}
		}

		public string getFormattedString(Dictionary<int,string> indexToTag)
		{
			List<string> patterns = new List<string>();
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < pin.Count; i++)
			{
				if(i>0)
					sb.Append(" v ");
				sb.Append("{");
				List<string> elements = new List<string>();


				HashSet<string> setin = new HashSet<string>(pin[i].Select(x => indexToTag != null && indexToTag.ContainsKey(x) ? "(" + x + "|" + indexToTag[x] + ")" : x + ""));
				List<HashSet<string>> setout = new List<HashSet<string>>();
				foreach (HashSet<int> s in pout[i])
				{
					setout.Add(new HashSet<string>(s.Select(x => indexToTag != null && indexToTag.ContainsKey(x) ? "(" + x + "|" + indexToTag[x] + ")" : x + "")));
				}


				if (setin.Count > 0)
					elements.Add("{" + Tools.listToString(setin).Replace(" ", ",") + "}");
				foreach (HashSet<string> set in setout)
				{
					elements.Add("!{" + Tools.listToString(set).Replace(" ", ",") + "}");
				}

				sb.Append(elements[0]);
				for (int j = 1; j < elements.Count; j++)
					sb.Append(" & " + elements[j]);
				patterns.Add("{"+sb.ToString()+"}");
				sb.Append("}");
			}

			return sb.ToString();
		}

		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();
			sb.AppendLine(base.ToString());
			sb.AppendLine("Patterns:;" + CountPatterns);
			sb.AppendLine("Attributes:;" + CountAttributes);
			sb.AppendLine("Total Pattern Length:;" + CountTotalPatternLength);
			sb.AppendLine("Patterns:;" + (CountTotalPatternLength*1.0/CountPatterns));
			sb.AppendLine("Complexity:;" + Complexity);
			List<string> elements = new List<string>();
			for (int i = 0; i < pin.Count; i++)
			{
				elements.Clear();
				HashSet<int> setin = pin[i];
				List<HashSet<int>> setout = pout[i];
				if (setin.Count > 0)
					elements.Add("{" + Tools.listToString(setin).Replace(" ", ",") + "}");
				foreach (HashSet<int> set in setout)
				{
					elements.Add("!{" + Tools.listToString(set).Replace(" ", ",") + "}");
				}
				sb.Append(elements[0]);
				for (int j = 1; j < elements.Count; j++)
					sb.Append(" & " + elements[j]);
				elements.Clear();
				sb.AppendLine();
			}
			return sb.ToString();
		}
	}


}
