﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{

	public class CommunityStamp : IEquatable<CommunityStamp>
	{
		int communityID;
		int[] vertices;
		private int hash;

		public int CommunityID
		{
			get
			{
				return communityID;
			}
		}

		public int Count
		{
			get
			{
				return vertices.Length;
			}
		}

		public int[] Vertices
		{
			get
			{
				return vertices;
			}
		}
		public CommunityStamp(int[] vertices, int communityID)
		{
			this.vertices = vertices;
			hash = 0;
			foreach(int v in vertices)
			{
				hash = hash ^ (v*v);
			}
			this.communityID = communityID;
		}

		public override int GetHashCode()
		{
			return hash;
		}



		public bool Equals(CommunityStamp cs)
		{
			if (hash != cs.hash)
				return false;
			if (vertices.Length != cs.vertices.Length)
				return false;
			for (int i = 0; i < vertices.Length; i++)
			{
				if (vertices[i] != cs.vertices[i])
					return false;
			}
			return true;
		}
	}

	public class CommunityStampSet
	{
		private Dictionary<CommunityStamp, Community> set;

		public CommunityStampSet()
		{
			set = new Dictionary<CommunityStamp, Community>(100000);
		}

		public void Add(Community c)
		{
			lock (set)
			{
				set[c.Stamp] = c;
			}
		}

		public void Add(CommunityStamp s,Community c)
		{
			lock (set)
			{
				set[s] = c;
			}
		}

		public Community getCommunity(CommunityStamp stamp)
		{
			Community c;
			lock (set)
			{
				c = set[stamp];
			}
			return c;
		}

		public bool Contains(Community c)
		{
			lock(set)
				return set.ContainsKey(c.Stamp);
		}

		public bool Contains(CommunityStamp s)
		{
			lock (set)
				return set.ContainsKey(s);
		}

		public int Count
		{
			get
			{
				lock (set)
				{
					return set.Count;
				}
			}
		}
	}
}
