﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class BitArray : BitClass
	{
		private int size;
		private int[] data;

		public BitArray(int n)
		{
			this.n = n;
			size = (n / 32) + ((n % 32 == 0) ? 0 : 1);
			data = new int[size];
		}

		public BitArray(string file)
		{
			StreamReader reader = new StreamReader(file);
			string line = reader.ReadLine();
			reader.Close();
			string[] parts = line.Split(';');
			
			//create data array
			n = int.Parse(parts[0]);
			size = (n / 32) + ((n % 32 == 0) ? 0 : 1);
			data = new int[size];

			//base number
			int b = int.Parse(parts[1]);

			//read
			string[] numbers = parts[2].Split(' ');
			for (int i = 0; i < size; i++)
			{
				data[i] = Convert.ToInt32(numbers[i], b);
			}
		}

		public BitArray(int[] data, int n)
		{
			size = (n / 32) + ((n % 32 == 0) ? 0 : 1);
			this.n = n;
			this.data = data;
		}

		public bool this[int i]
		{
			get
			{
				return get(i);
			}
			set
			{
				set(i, value);
			}
		}

		public bool get(int i)
		{
			return (1 << (31 - (i % 32)) & data[i / 32]) != 0;
		}

		public void set(int i, bool v)
		{
			if (v)
				data[i / 32] = (1 << (31 - (i % 32))) | data[i / 32];
			else
				data[i / 32] = ~(1 << (31 - (i % 32))) & data[i / 32];
		}

		public bool contains(HashSet<int> set)
		{
			foreach (int i in set)
			{
				if (!get(i))
					return false;
			}
			return true;
		}

		


		public int countValue(bool v)
		{
			if (!v)
				return n - countValue(true);
			int count = 0;
			for (int i = 0; i < data.Length-1; i++)
			{
				if (data[i] != 0)
				{
					count += NumberOfSetBits(data[i]);
				}
			}
			for (int i = 0; i < 32; i++)
			{
				if (get((data.Length - 1) * 32 + i))
					count++;
			}
			return count;
		}

		public int[] getIndicesByValue(bool v)
		{
			int[] indices = new int[countValue(v)];
			int count = 0;

			if (v)
			{
				for (int i = 0; i < data.Length; i++)
				{
					if (data[i] != 0)
					{
						for (int j = i * 32; j < (i + 1) * 32 && j<n; j++)
						{
							if (get(j))
								indices[count++] = j;
						}
					}
				}
			}
			else
			{
				for (int i = 0; i < data.Length; i++)
				{
					if (data[i] == 0)
					{
						for (int j = i * 32; j < (i + 1) * 32 && j < n; j++)
						{
							indices[count++] = j;
						}
					}
					else
					{
						for (int j = i * 32; j < (i + 1) * 32 && j < n; j++)
						{
							if (!get(j))
								indices[count++] = j;
						}
					}
				}
			}
			return indices;
		}

		public List<int> getIndexsByValue(bool v, int cap = 0)
		{
			List<int> indexes;

			if (cap == 0)
				indexes = new List<int>();
			else
				indexes = new List<int>(countValue(v));
			
			if (v)
			{
				for (int i = 0; i < data.Length; i++)
				{
					if (data[i] != 0)
					{
						for (int j = i * 32; j < (i + 1) * 32 && j<n; j++)
						{
							if (get(j))
								indexes.Add(j);
						}
					}
				}
			}
			else
			{
				for (int i = 0; i < data.Length; i++)
				{
					if (data[i] == 0)
					{
						for (int j = i * 32; j < (i + 1) * 32 && j < n; j++)
						{
							indexes.Add(j);
						}
					}
					else
					{
						for (int j = i * 32; j < (i + 1) * 32 && j<n; j++)
						{
							if (!get(j))
							{
								indexes.Add(j);
							}
						}
					}
				}
			}
			return indexes;

		}

		public string toARFLine()
		{
			List<string> values = new List<string>();
			for (int j = 0; j < n; j++)
			{
				if (get(j))
					values.Add(j + " 1");
			}
			return "{" + string.Join(",", values.ToArray())+"}";
		}

		public double JaccardDistance(BitArray o)
		{
			int m00 = 0;
			int m01 = 0;
			int m10 = 0;
			int m11 = 0;
			for (int i = 0; i < n; i++)
			{
				if (!get(i))
				{
					if (!o.get(i))
						m00++;
					else
						m01++;
				}
				else
				{
					if (!o.get(i))
						m10++;
					else
						m11++;
				}
			}
			double jd = ((double)(m01 + m10)) / ((double)(m01 + m10 + m11));
			return jd;
		}

		public int[] Data
		{
			get
			{
				return data;
			}
		}

		public int NumInts
		{
			get
			{
				return size;
			}
		}

		public void Write(string file, int b)
		{
			StreamWriter writer = new StreamWriter(file);
			writer.Write(ToString(b));
			writer.Close();
		}

		public void Write(string file)
		{
			Write(file, 10);
		}

		public string ToString(int b)
		{
			StringBuilder sb = new StringBuilder();
			sb.Append(n + ";"+b+";");
			for (int i = 0; i < data.Length; i++)
			{
				sb.Append(format(data[i], b));
				if (i < data.Length - 1)
					sb.Append(" ");
			}
			return sb.ToString();
		}

		public override string ToString()
		{
			return ToString(2);
		}
	}
}
