﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{

	public class Community : IComparable<Community>
	{
		//TODO temp
		public RemineClassifier classifier;

		public int numOp = 0;
		/**
		 * All vertices in this community with the number of internal links to it
		 */
		Dictionary<int, int> vertices;

		/**
		 * All neighbours to this community with the number of links to it
		 */
		Dictionary<int, int> neighbours;

		/**
		 * The graph structure
		 */
		UndirectedGraph graph;

		public static QualityMeasure measure = new CommunityGain(true);

		public int treeSize;

		static Dictionary<CommunityStamp, double> mod = new Dictionary<CommunityStamp,double>();

		public int ID
		{
			get
			{
				return id;
			}
		}

		private int id;
		private static int idCounter = 1 ;

		CommunityStamp stamp;
		
		private CommunityStamp getStamp()
		{
			int[] stamp = new int[vertices.Count];
			int i = -1;
			foreach (int v in vertices.Keys)
				stamp[++i] = v;
			Array.Sort(stamp);
			return new CommunityStamp(stamp, id);
		}

		public double JaccardDistance(Community c)
		{
			HashSet<int> a = new HashSet<int>(this.vertices.Keys);
			HashSet<int> b = new HashSet<int>(c.vertices.Keys);
			HashSet<int> union = new HashSet<int>(a.Union(b));
			return (1.0 * (union.Count - a.Intersect(b).Count()))/union.Count;
		}


		public Community(UndirectedGraph graph)
		{
			this.id = idCounter++;
			this.graph = graph;
			vertices = new Dictionary<int, int>();
			neighbours = new Dictionary<int, int>();
		}

		public Community(UndirectedGraph graph, string vstring) : this(graph)
		{
			foreach (string v in vstring.Trim().Split(' '))
			{
				AddVertice(int.Parse(v));
			}
		}

		public Community(UndirectedGraph graph, CommunityStamp stamp): this(graph)
		{
			foreach (int v in stamp.Vertices)
			{
				AddVertice(v);
			}
			this.id = stamp.CommunityID;
		}

		public BitArray getVerticeMembership()
		{
			BitArray ba = new BitArray(graph.Count);
			foreach (int v in vertices.Keys)
				ba.set(v, true);
			return ba;
		}

		/**
		 * Compare two communities
		 */
		public bool Equals(Community c)
		{
			if (c.Size != Size)
				return false;
			foreach (int v in vertices.Keys)
			{
				if (!c.ContainsVertice(v))
					return false;
			}
			return true;
		}

		/**
		 * Compare two communities, the highest one has the highest community gain. If the
		 * community gain is equal the size is used.
		 */
		public int CompareTo(Community c)
		{
			if (c == null)
				return 1;
			double cgOwn = measure.communityGain(this);
			double cgOth = measure.communityGain(c);
			if (cgOwn > cgOth)
				return -1;
			else if (cgOwn < cgOth)
				return 1;

			if (this.Size > c.Size)
				return -1;
			else if (this.Size < c.Size)
				return 1;
			return 0;
		}

		public int Size
		{
			get
			{
				return vertices.Count;
			}
		}

		public Dictionary<int, int> getNeighbours()
		{
			return neighbours;
		}

		public Dictionary<int, int> getVertices()
		{
			return vertices;
		}

		public bool ContainsVertice(int v)
		{
			return vertices.ContainsKey(v);
		}

		public void AddVertice(string s)
		{
			string[] vertices = s.Split(',');
			foreach(string vertex in vertices)
			{
				string[] numbers = vertex.Split('#');
				AddVertice(int.Parse(numbers[0]));
			}
		}

		public void AddVertice(int v)
		{
			stamp = null;
			//allready in community
			if (vertices.ContainsKey(v))
				return;

			//insert and delete from lists
			neighbours.Remove(v);
			vertices.Add(v, 0);

			//update neighbour list (number of type B errors per node)
			foreach (int u in graph.getNeighbours(v))
			{
				if (v == u) //ignore edges to node itself
					continue;

				if (vertices.ContainsKey(u)) // no type B error
				{
					vertices[u]++;
					vertices[v]++;
					continue;
				}

				//insert if not included, add 1 otherwise
				if (neighbours.ContainsKey(u))
					neighbours[u] = neighbours[u] + 1;
				else
					neighbours.Add(u, 1);
			}
		}

		public void RemoveVertice(int v)
		{
			stamp = null;
			//not in list
			if (!vertices.ContainsKey(v))
				return;

			//remove from vertex list
			vertices.Remove(v);

			//update neigbours list (type B errors)
			foreach (int u in graph.getNeighbours(v))
			{
				if (v == u) //ignore edges to node itself
					continue;

				if (vertices.ContainsKey(u))
				{
					vertices[u]--; //internal edge less for node u

					//extra external edge for neighbour
					if (neighbours.ContainsKey(v))
						neighbours[v]++;
					else
						neighbours[v] = 1;
				}
				else
				{
					neighbours[u]--;
					if (neighbours[u] == 0)
						neighbours.Remove(u);
				}
			}
		}

		public List<Operation> getOperations()
		{
			//List<Operation> opr = new List<Operation>();
			//int key=-1,value=-1;
			//foreach (KeyValuePair<int, int> kv in neighbours)
			//{
			//    if (kv.Value > value)
			//    {
			//        key = kv.Key;
			//        value = kv.Value;
			//    }
			//}
			//opr.Add(new Operation(key, measure.gain(this, key, true), true));

			//key = -1;
			//value = int.MaxValue;
			//foreach (KeyValuePair<int, int> kv in vertices)
			//{
			//    if (kv.Value < value)
			//    {
			//        key = kv.Key;
			//        value = kv.Value;
			//    }
			//}
			//opr.Add(new Operation(key, measure.gain(this, key, false), false));
			//return opr;


			//old
			List<Operation> operations = new List<Operation>(vertices.Count+neighbours.Count);
			Operation o;
			foreach (int u in vertices.Keys)
			{
				o = new Operation(u, measure.gain(this, u, false), false);
				operations.Add(o);
			}
			foreach (int u in neighbours.Keys)
			{
				o = new Operation(u, measure.gain(this, u, true), true);
				operations.Add(o);
			}
			return operations;
		}

        public bool improve()
        {
            Operation max = getOperations().Max();
            if (max != null && max.isImprovement())
            {
                apply(max);
                numOp++;
                return true;
            }
            return false;
        }

		public void apply(Operation o)
		{
			if (o.add)
				AddVertice(o.v);
			else
				RemoveVertice(o.v);
		}

		public UndirectedGraph getGraph()
		{
			return graph;
		}

		public double communityGain()
		{
			return measure.communityGain(this);
		}


		public override string ToString()
		{
            if (vertices.Count == 0)
                return "";
			string line = "";
			foreach (KeyValuePair<int, int> kv in vertices.OrderBy(key => key.Key))
			{
				line +=kv.Key+" ";
			}
			return line.Substring(0, line.Length - 1);
		}

		public double intraClusterDensity()
		{
			double edgesIternal = 0;
			foreach (KeyValuePair<int, int> kv in vertices)
			{
				edgesIternal += 0.5 * kv.Value;
			}
			double edgesMaximal = (Size * (Size - 1)) * 0.5;
			return edgesIternal / edgesMaximal;
		}

		public double conductance()
		{
			double edgesIternal = 0;
			foreach (KeyValuePair<int, int> kv in vertices)
			{
				edgesIternal += 0.5 * kv.Value;
			}

			double edgesExternal = 0;
			foreach (KeyValuePair<int, int> kv in neighbours)
			{
				edgesExternal += kv.Value;
			}

			return edgesExternal / (2 * edgesIternal + edgesExternal);
		}

		public double inverseConductance()
		{
			double edgesIternal = 0;
			double degreeSum = 0;
			foreach (KeyValuePair<int, int> kv in vertices)
			{
				degreeSum += graph.degree(kv.Key);
				edgesIternal += kv.Value;
			}
			return edgesIternal / degreeSum;
		}

		public CommunityStamp Stamp
		{
			get
			{
				if (stamp == null)
					stamp = getStamp();
				return stamp;
			}
		}

		public double modularity()
		{
			lock (mod)
			{
				if (mod.ContainsKey(Stamp))
					return mod[Stamp];
			}

			double sum = 0;
			foreach(int i in vertices.Keys)//for (int i = 0; i < graph.Count; i++)
			{
				foreach (int j in vertices.Keys)//for (int j = 0; j < graph.Count; j++)
				{
					if (vertices.ContainsKey(i) == vertices.ContainsKey(j))
					{
						double aij = (graph.isEdge(i, j) || i==j) ? 1 : 0;
						double pij = (graph.degree(i) * graph.degree(j)) / (2.0 * graph.NumEdges);
						sum += aij - pij;
					}
				}
			}
			lock (mod)
			{
				mod[Stamp] = sum / (2.0 * graph.NumEdges);
				return mod[Stamp];
			}

		}
	}
}
