﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Globalization;

namespace CommunityDetection
{
	
	public class TestConjunctionClassifier : ExpProgram
	{
		public CandidateFileGenerator generator;
		BinaryMatrix matrix;
		string datafile;
		ThreadSafeQueue<KeyValuePair<int,double>> jobqueue;
		List<Thread> threads;
		int numThreads;
		public StreamWriter jobwriter;
		HashSet<int> trueRows;

		public override void loadArguments(string[] args)
		{
			generator = new CandidateFileGenerator(datadir+args[0]);
			datafile = datadir+args[1];
			List<int> mpl = new List<int>();
			List<double> alphas = new List<double>();

			numThreads = int.Parse(args[2]);

			string[] parts;
			parts = args[3].Split(' ');
			foreach (string s in parts)
				mpl.Add(int.Parse(s));

			parts = args[4].Split(' ');
			foreach (string s in parts)
				alphas.Add(double.Parse(s, CultureInfo.InvariantCulture));

			jobqueue = new ThreadSafeQueue<KeyValuePair<int,double>>();

			foreach (int mp in mpl)
			{
				foreach (double a in alphas)
				{
					KeyValuePair<int, double> job = new KeyValuePair<int, double>(mp, a);
					jobqueue.Enqueue(job);
				}
			}

			//Console.WriteLine("Queue size: "+jobqueue.Count);

		}

		public override void start()
		{
			jobwriter = new StreamWriter(expdir + expName + ".csv");
			String s = ";";
			String line = "Maximum pattern length" + s + "alpha";
			line += s + "Duration (s)" + s + "Number of patterns inA" + s + "Number of patterns outA" + s + "TP" + s + "FP" + s + "TN" + s + "FN";
			line += s + "Precision" + s + "Recall"+s+"g-Measure";
			explog(line);

			log("Candidatesfile: " + datafile);

			Console.Write("Reading description data.. ");
			matrix = new BinaryMatrixBitArrays(datafile);
			Console.WriteLine("Done!");

			log("Description data: " + matrix.M + "x" + matrix.N);
			Console.Write("Creating community");
			ThreadSafeQueue<Community> queue = generator.generate(graph,matrix);
			Community c = queue.Dequeue();
			Console.WriteLine("Intra cluster density C:"+ c.intraClusterDensity());
			log("Intra cluster density C:" + c.intraClusterDensity());
			log("CG(c): " + c.communityGain());
			Console.WriteLine("CG(c):" + c.communityGain());
			trueRows = new HashSet<int>(c.getVertices().Keys);
			Console.WriteLine("Done! Community: " + c.Size);
			log("Community size: " + c.Size);

			Console.Write("Creating threads.. ");
			threads = new List<Thread>();
			for (int i = 0; i < numThreads; i++)
			{
				Thread t = new Thread(this.run);
				threads.Add(t);
				t.Start();
			}
			log("Num threads: " + numThreads);
			Console.WriteLine("Done!");
			
			foreach (Thread t in threads)
				t.Join();
			
			jobwriter.Close();
		}


		public void run()
		{
			while (!jobqueue.Empty())
			{
				KeyValuePair<int, double> job;
				try
				{
					job = jobqueue.Dequeue();
				}
				catch (Exception ) //empty jopqueue
				{
				    return;
				}

				int threadId = 0;
				foreach (Thread t in threads)
				{
					if (t == Thread.CurrentThread)
						break;
					threadId++;
				}
				Console.WriteLine("Thread " + threadId + " starts on maxPatternLength:" + job.Key + ",  alpha: "+job.Value);
				TimeMeasure measure = new TimeMeasure();
				ConjunctionBuilder builder = new ConjunctionBuilder(matrix, job.Key, job.Value);
				ConjunctionClassifier classifier = (ConjunctionClassifier) builder.build(new HashSet<int>(trueRows));
				double duration = measure.getDuration();
				int[] q = classifier.quality(matrix, trueRows);
				String s = ";";
				String line = job.Key+s+job.Value;
				line += s + duration + s + classifier.inA.Count + s+classifier.outA.Count + s + q[0] + s + q[1] + s + q[2] + s + q[3];
				line += s + classifier.precision(q) + s + classifier.recall(q) + s+builder.measure(q[0], q[1], q[2], q[3]);
				explog(line);
				Console.WriteLine("Thread " + threadId + " finished in " + duration + " seconds");
				Console.WriteLine("Queue size: " + jobqueue.Count);
			}
		}
	
		public void explog(string msg)
		{
			lock(jobwriter)
			{
				jobwriter.WriteLine(msg);
				jobwriter.Flush();
			}
		}
	}
}
